
# q-expansion of newform 2040.2.a.d, downloaded from the LMFDB on 22 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2040
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [1], [-4], [4], [2], [1], [-4], [0], [-2], [0], [-2], [6], [-8], [8], [6], [0], [14], [0], [12], [-2], [8], [4], [2], [14], [18], [0], [-4], [14], [-6], [8], [-12], [-22], [12], [2], [8], [-14], [20], [-8], [-18], [0], [6], [8], [-2], [22], [-8], [12], [8], [-12], [22], [26], [-16], [2], [0], [-14], [24], [-18], [0], [22], [-14], [20], [30], [-32], [-28], [-10], [-2], [-4], [22], [12], [-10], [-14], [-16], [-36], [-30], [12], [0], [-30], [-26], [-10], [-6], [20], [-10], [28], [-22], [-32], [-4], [6], [18], [18], [-24], [-4], [-12], [-20], [-8], [-4], [16], [-22], [-18], [16], [-18], [28], [30], [-12], [-6], [44], [26], [-20], [-14], [32], [18], [-12], [2], [-22], [36], [32], [-42], [-44], [40], [14], [0], [46], [-2], [6], [36], [28], [2], [38], [20], [-16], [-38], [4], [24], [40], [10], [-22], [-14], [6], [20], [-42], [-10], [-36], [-50], [20], [-12], [6], [-12], [-10], [42], [-36], [24], [-18], [-26], [-24], [0], [44], [20], [56], [-34], [2], [54], [52], [-6], [8], [-40], [-30], [-24], [16], [14], [-46], [6], [0], [14], [-28], [-14], [-24], [22], [-28], [30], [40], [30], [-56], [-44], [-34], [-6], [56], [-30], [38], [40], [-62], [-12], [46], [44], [20], [-30], [-52], [18], [-38], [22], [-6], [-24], [-26], [-32], [-14], [50], [32], [-42], [32], [-36], [-18], [28], [22], [10], [4], [28], [36], [-62], [8], [42], [16], [62], [22], [-40], [18], [-44], [-44], [-10], [-30], [4], [-16], [-60], [34], [60], [-56], [10], [-8], [-40], [-38], [-26], [-20], [-32], [36], [-20], [-24], [-26], [66], [36], [-76], [-36], [44], [72], [50], [62], [8], [-78], [-50], [24], [-58], [4], [70], [-14], [36], [12], [38], [46], [58], [60], [-70], [26], [44], [-10], [78], [24], [-46], [32], [26], [-56], [-52], [-10], [42], [-48], [-16], [0], [0], [-26], [-52], [-8], [46], [22], [32], [-14], [78], [68], [-6], [28], [38], [30], [-64], [-82], [20], [56], [10], [14], [72], [-20], [4], [70], [28], [14], [-72], [-54], [-80], [-26], [14], [-72], [0], [42], [-40], [-76], [-66], [34], [4], [62], [-70], [-8], [26], [-14], [-28], [44], [-88], [54], [-50], [-2], [-80], [-36], [60], [-76], [46], [-46], [10], [0], [-78], [-86], [78], [-8], [46], [92], [-26], [80], [-84], [6], [36], [-42], [10], [-84], [-2], [-54], [-40], [52], [-6], [24], [78], [-42], [-24], [-28], [64], [-30], [-50], [-32], [2], [-80], [4], [-24], [-46], [40], [94], [-52], [-4], [50], [34], [98], [78], [90], [-60], [18], [-100], [88], [64], [58], [-28], [96], [10], [6], [-72], [56], [-24], [-34], [24], [-70], [44], [-30], [94], [66], [8], [-62], [-6], [-16], [2], [-78], [-64], [-44], [14], [46], [12], [20], [10], [38], [-84], [-20], [42], [-96], [66], [-10], [24], [16], [38], [-42], [-36], [-74], [-68], [44]]
